<?php

/**
 * Created by PhpStorm.
 * User: crose
 * Date: 1/26/16
 * Time: 2:56 PM
 */

namespace qfq;

require_once(__DIR__ . '/../../qfq/helper/HelperFormElement.php');
require_once(__DIR__ . '/../../qfq/Constants.php');

use PHPUnit\Framework\TestCase;

//class HelperFormElementTest extends \PHPUnit_Framework_TestCase {

/**
 * Class HelperFormElementTest
 * @package qfq
 */
class HelperFormElementTest extends TestCase {

    public function testExplodeParameter() {
        $a = array();

        HelperFormElement::explodeParameter($a, FE_PARAMETER);
        $this->assertEquals(array(), $a, "Both arrays should be equal and empty");

        $a = ['id' => 100, 'parameter' => "work=12\nlist=1:new,2:edit,3:delete"];
//        $b = ['id' => 100, 'parameter' => "work=12\nlist=1:new,2:edit,3:delete", 'work' => 12, 'list' => '1:new,2:edit,3:delete'];
        $b = ['id' => 100, 'parameter' => "", 'work' => 12, 'list' => '1:new,2:edit,3:delete'];

        HelperFormElement::explodeParameter($a, FE_PARAMETER);
        $this->assertEquals($b, $a, "Both arrays should be equal");
    }

    public function testExplodeParameterInArrayElements() {
        $a = array();
        HelperFormElement::explodeParameterInArrayElements($a, FE_PARAMETER);
        $this->assertEquals(array(), $a, "Both arrays should be equal and empty");

        $a = [0 => ['id' => 100, 'parameter' => "work=12\nlist=1:new,2:edit,3:delete"]];
//        $b = [0 => ['id' => 100, 'parameter' => "work=12\nlist=1:new,2:edit,3:delete", 'work' => 12, 'list' => '1:new,2:edit,3:delete']];
        $b = [0 => ['id' => 100, 'parameter' => "", 'work' => 12, 'list' => '1:new,2:edit,3:delete']];

        HelperFormElement::explodeParameterInArrayElements($a, FE_PARAMETER);
        $this->assertEquals($b, $a, "Both arrays should be equal");
    }


    /**
     * @expectedException \qfq\UserFormException
     *
     */
    public function testExplodeFieldParameterException() {

        $a = [0 => ['id' => 111, 'name' => 'hello', 'collision' => 'dummy', 'parameter' => "work=12\ncollision=1:new,2:edit,3:delete"]];
        HelperFormElement::explodeParameterInArrayElements($a, FE_PARAMETER);

    }

    /**
     *
     */
    public function testDuplicateRetypeElements() {

        $a = [0 => [FE_NAME => 'hello', FE_LABEL => 'my label', FE_NOTE => 'my note']];
        $b = HelperFormElement::duplicateRetypeElements($a);
        $this->assertEquals($a, $b, "Both arrays should be equal");

        $a = [0 => [FE_NAME => 'hello', FE_LABEL => 'my label', FE_NOTE => 'my note', FE_RETYPE => '']];
        $b = HelperFormElement::duplicateRetypeElements($a);
        $this->assertEquals(count($a) + 1, count($b), "Both arrays should be same count");
        $this->assertEquals('1', $b[0][FE_RETYPE], "FE_RETYPE should be set to '1'");

        $a = [0 => [FE_NAME => 'hello', FE_LABEL => 'my label', FE_NOTE => 'my note', FE_RETYPE => '0']];
        $b = HelperFormElement::duplicateRetypeElements($a);
        $this->assertEquals($a, $b, "Both arrays should be same count");
        $this->assertEquals('0', $b[0][FE_RETYPE], "FE_RETYPE should be unchanged");

        $a = [0 => [FE_NAME => 'hello', FE_LABEL => 'my label', FE_NOTE => 'my note', FE_RETYPE => '1']];
        $b = HelperFormElement::duplicateRetypeElements($a);
        $this->assertEquals(count($a) + 1, count($b), "Both arrays should be same count");
        $this->assertEquals('1', $b[0][FE_RETYPE], "FE_RETYPE should be unchanged");
        $this->assertEquals($b[0][FE_NAME] . RETYPE_FE_NAME_EXTENSION, $b[1][FE_NAME], "New name should be extended by: " . RETYPE_FE_NAME_EXTENSION);
        $this->assertEquals($b[0][FE_LABEL], $b[1][FE_LABEL], "Both text should be equal");
        $this->assertEquals($b[0][FE_NOTE], $b[1][FE_NOTE], "Both text should be equal");

        $a = [0 => [FE_NAME => 'hello', FE_LABEL => 'my label', FE_NOTE => 'my note', FE_RETYPE => '1', FE_RETYPE_LABEL => '', FE_RETYPE_NOTE => '']];
        $b = HelperFormElement::duplicateRetypeElements($a);
        $this->assertEquals('', $b[1][FE_LABEL], "String should be empty");
        $this->assertEquals('', $b[1][FE_NOTE], "String should be empty");

        $a = [0 => [FE_NAME => 'hello', FE_LABEL => 'my label', FE_NOTE => 'my note', FE_RETYPE => '1', FE_RETYPE_LABEL => 'new label', FE_RETYPE_NOTE => 'new note']];
        $b = HelperFormElement::duplicateRetypeElements($a);

        $this->assertEquals('my label', $b[0][FE_LABEL], "String should be equal");
        $this->assertEquals('my note', $b[0][FE_NOTE], "String should be equal");
        $this->assertEquals('new label', $b[1][FE_LABEL], "String should be equal");
        $this->assertEquals('new note', $b[1][FE_NOTE], "String should be equal");

    }

    /**
     *
     */
    public function testInitActionFormElement() {

        $list = [FE_TYPE, FE_SLAVE_ID, FE_SQL_VALIDATE, FE_SQL_BEFORE, FE_SQL_INSERT, FE_SQL_UPDATE, FE_SQL_DELETE,
            FE_SQL_AFTER, FE_EXPECT_RECORDS, FE_REQUIRED_LIST, FE_MESSAGE_FAIL, FE_SENDMAIL_TO, FE_SENDMAIL_CC,
            FE_SENDMAIL_BCC, FE_SENDMAIL_FROM, FE_SENDMAIL_SUBJECT, FE_SENDMAIL_REPLY_TO, FE_SENDMAIL_FLAG_AUTO_SUBMIT,
            FE_SENDMAIL_GR_ID, FE_SENDMAIL_X_ID, FE_SENDMAIL_X_ID2, FE_SENDMAIL_X_ID3];

        $expect = array();
        foreach ($list as $key) {
            $expect[$key] = '';
        }

        // Complete empty
        $result = HelperFormElement::initActionFormElement(array());
        $this->assertEquals($expect, $result, "Both arrays should be equal");

        // Two known elements
        $fe = array();
        $fe[FE_TYPE] = FE_TYPE_AFTER_DELETE;
        $fe[FE_SENDMAIL_TO] = 'info@example.com';
        $expect[FE_TYPE] = $fe[FE_TYPE];
        $expect[FE_SENDMAIL_TO] = $fe[FE_SENDMAIL_TO];
        $result = HelperFormElement::initActionFormElement($fe);
        $this->assertEquals($expect, $result, "Both arrays should be equal");

        $fe['unknown'] = 'unknown';
        $expect['unknown'] = 'unknown';
        $result = HelperFormElement::initActionFormElement($fe);
        $this->assertEquals($expect, $result, "Both arrays should be equal");
    }

    /**
     *
     */
    public function testInitUploadFormElement() {

        $list = [FE_SQL_BEFORE, FE_SQL_INSERT, FE_SQL_UPDATE, FE_SQL_DELETE, FE_SQL_AFTER];

        $expect = array();
        foreach ($list as $key) {
            $expect[$key] = '';
        }

        // Complete empty
        $result = HelperFormElement::initUploadFormElement(array());
        $this->assertEquals($expect, $result, "Both arrays should be equal");

        // Two known elements
        $fe = array();
        $fe[FE_SQL_BEFORE] = 'SELECT willi';
        $fe[FE_SQL_AFTER] = 'UPDATE jane';
        $expect[FE_SQL_BEFORE] = $fe[FE_SQL_BEFORE];
        $expect[FE_SQL_AFTER] = $fe[FE_SQL_AFTER];
        $result = HelperFormElement::initUploadFormElement($fe);
        $this->assertEquals($expect, $result, "Both arrays should be equal");

        $fe['unknown'] = 'unknown';
        $expect['unknown'] = 'unknown';
        $result = HelperFormElement::initUploadFormElement($fe);
        $this->assertEquals($expect, $result, "Both arrays should be equal");
    }

    public function testTgGetMaxLength() {
        $result = HelperFormElement::tgGetMaxLength('');
        $this->assertEquals(FE_TEMPLATE_GROUP_DEFAULT_MAX_LENGTH, $result, "Expect Default");

        $result = HelperFormElement::tgGetMaxLength(0);
        $this->assertEquals(FE_TEMPLATE_GROUP_DEFAULT_MAX_LENGTH, $result, "Expect Default");

        $result = HelperFormElement::tgGetMaxLength('0');
        $this->assertEquals(FE_TEMPLATE_GROUP_DEFAULT_MAX_LENGTH, $result, "Expect Default");

        $result = HelperFormElement::tgGetMaxLength('1');
        $this->assertEquals('1', $result, "Expect Default");

        $result = HelperFormElement::tgGetMaxLength('10');
        $this->assertEquals('10', $result, "Expect Default");

        $result = HelperFormElement::tgGetMaxLength(10);
        $this->assertEquals('10', $result, "Expect Default");

    }
//    public function testExplodeTemplateGroupElements() {
//        $formElements = array();
//
//        $result = HelperFormElement::explodeTemplateGroupElements(array());
//        $this->assertEquals(array(), $result, "Both arrays should be equal");
//
//        $formElements = [[FE_NAME => 'nameA', FE_LABEL => 'labelA', FE_VALUE => '1'], [FE_NAME => 'nameB', FE_LABEL => 'labelB', FE_VALUE => '11']];
//        $result = HelperFormElement::explodeTemplateGroupElements($formElements);
//        $this->assertEquals($formElements, $result, "Both arrays should be equal");
//
//        $formElements = [[FE_NAME => 'nameA', FE_LABEL => 'label' . FE_TEMPLATE_GROUP_NAME_PATTERN . 'A', FE_VALUE => '1'], [FE_NAME => 'nameB', FE_LABEL => 'labelB', FE_VALUE => '11']];
//        $result = HelperFormElement::explodeTemplateGroupElements($formElements);
//        $this->assertEquals($formElements, $result, "Both arrays should be equal");
//
//        $formElements = [[FE_NAME => 'nameA', FE_LABEL => 'label' . FE_TEMPLATE_GROUP_NAME_PATTERN . 'A', FE_VALUE => '1', NAME_TG_COPIES => 2], [FE_NAME => 'nameB', FE_LABEL => 'labelB', FE_VALUE => '11']];
//        $result = HelperFormElement::explodeTemplateGroupElements($formElements);
//        $formElements = [[FE_NAME => 'nameA', FE_LABEL => 'label1A', FE_VALUE => '1'],
//            [FE_NAME => 'nameA', FE_LABEL => 'label2A', FE_VALUE => '1'],
//            [FE_NAME => 'nameB', FE_LABEL => 'labelB', FE_VALUE => '11']];
//        $this->assertEquals($formElements, $result, "Both arrays should be equal");
//
//        $formElements = [[FE_NAME => FE_TEMPLATE_GROUP_NAME_PATTERN . 'nameA', FE_LABEL => 'label' . FE_TEMPLATE_GROUP_NAME_PATTERN . 'A', FE_VALUE => '1', NAME_TG_COPIES => 2], [FE_NAME => 'nameB', FE_LABEL => 'labelB', FE_VALUE => '11']];
//        $result = HelperFormElement::explodeTemplateGroupElements($formElements);
//        $formElements = [[FE_NAME => '1nameA', FE_LABEL => 'label1A', FE_VALUE => '1'],
//            [FE_NAME => '2nameA', FE_LABEL => 'label2A', FE_VALUE => '1'],
//            [FE_NAME => 'nameB', FE_LABEL => 'labelB', FE_VALUE => '11']];
//        $this->assertEquals($formElements, $result, "Both arrays should be equal");
//    }
}
